package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : build_log_parser Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// Class for reading rule types from configuration file to RuleTypes object.
//
//------------------------------------------------------------------------------

import lib_groovy.RuleTypes
import lib_groovy.ZfReport;

class RuleConfigReader{
	
	/**
	 * Reads rule types from configuration file
	 * 
	 * @param configFileText - content of the configuration file
	 * 
	 * @return RuleTypes object
	 */
	public static RuleTypes readFromXml(String configFileText) throws Exception{
		RuleTypes configData = RuleTypes.getInstance();
		
		def rootTypes = new XmlSlurper().parseText(configFileText);
		
		String log = "";
		for (xmlnode in rootTypes.Types.Type) {
			String name = xmlnode.@name;
			String color = xmlnode.@color;
			String errorFlag = xmlnode.@errorFlag;
			int priority = 0;
			
			try {
				priority = Integer.parseInt((String)xmlnode.@priority);
			} catch(Exception e) {}
			
			
			if ((name == null) || name.isEmpty() || (color == null) || color.isEmpty() || priority == 0) {
				String exMessage = "iterateParamXmlNodes() in config.groovy: param does not have expected format.";
				ZfReport.error(exMessage);
			} else {
				log += "  name = ${name} color = ${color} priority = ${priority}\n";
				if(errorFlag == null || errorFlag.isEmpty() || !errorFlag.equalsIgnoreCase("true")) {
					configData.addType(name.toLowerCase(), color.toLowerCase(), priority);
				} else {
					configData.addType(name.toLowerCase(), color.toLowerCase(), priority, true);
					log += " errorFlag = ${errorFlag}";
				}
				
				log += "\n";
			}
			
		}
		
		//ZfReport.info("=== Types ===\n$log");
		
		return configData;
	}
	
	
}