package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : build_log_parser Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// This class contains ArrayList of rule types
//
//------------------------------------------------------------------------------

import lib_groovy.RuleType;

public class RuleTypes {
	
	//ArrayList of rule types
	private static final ArrayList<RuleType> types = new ArrayList<RuleType>();

	private static RuleTypes instance = null;
	
	private RuleTypes(){ }
	
	public static RuleTypes getInstance() {
		if(instance == null) {
			instance = new RuleTypes();
		}
		
		return instance;
	}
	
	/**
	 * Adds new type to ArrayList, if does not exist
	 * 
	 * @param name - name of the rule type
	 * @param color - color for references
	 * @param priority - priority for rule
	 * @param errorFlag - if this item error
	 */
	public void addType(String name, String color, int priority, boolean errorFlag = false) {
		RuleType isExist = getRuleTypeByName(name);
		
		if(isExist != null) {return;}
		
		RuleType type;
		
		if(errorFlag) {
			type = new RuleType(name, color, priority, errorFlag);
		} else {
			type =  new RuleType(name, color, priority);
		}
		
		types.add(type);
		
	}
	
	/**
	 * Get rule type by that name
	 * 
	 * @param name - rule type name
	 * 
	 * @return rule type object
	 */
	private RuleType getRuleTypeByName(String name) {
		for(RuleType type: types) {
			if(type.getName().equals(name.toLowerCase())) {
				return type;
			}
		}
		
		return null;
	}
	
	/**
	 * Sort types by priority and return
	 * 
	 * @return types
	 */
	public ArrayList<RuleType> getSortedTypes() {
		Collections.sort(types);
		
		return types;
	}

	
	
	
}