package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : build_log_parser Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// Class for reading rules xml file. 
//
//------------------------------------------------------------------------------

import lib_groovy.Rule
import lib_groovy.RuleType
import lib_groovy.RuleTypes
import lib_groovy.ZfReport;

class RulesParser{
	
	//list of lines rules
	private ArrayList<Rule> rulesQueue = new ArrayList<Rule>();
	//list of delimiters
	private ArrayList<Delimiter> delimQueue = new ArrayList<Delimiter>();
	
	public RulesParser(){}
	
	
	/**
	 * Reads rules file and creates rules and delimiter Queues
	 *
	 * @param path - to rules file
	 * @param types - types of rules
	 */
	public void parse(String path, RuleTypes types) {
		
		def rules = new XmlSlurper().parseText(new File(path).text);
		
		
		for (delimiters in rules.Rules.Delimiters) {
			for (delimiter in delimiters.Delimiter) {
				Delimiter currDelim = new Delimiter((String)delimiter.@value);
				//ZfReport.info("Delimiter created: " + (String)delimiter.@value);
				
				for (rule in delimiter.Rule) {
					Rule currRule = createRule((String)rule.@type, (String)rule, types);
					if(currRule == null) {
						continue;
					}
					currDelim.setRule(currRule);
					//ZfReport.info("Rule added to delimeter. Type - " + (String)rule.@type + ". Rule - " + (String)rule);
				}
				
				delimQueue.add(currDelim);
			}
			
			for (rule in delimiters.Commons.Rule) {
				Rule currRule = createRule((String)rule.@type, (String)rule, types);
				if(currRule == null) {
					continue;
				}
				
				for(Delimiter delimiter: delimQueue) {
					delimiter.setRule(currRule);
				}
				
				//ZfReport.info("Rule added to each delimeter. Type - " + (String)rule.@type + ". Rule - " + (String)rule);
			}
		}
		
		for (lines in rules.Rules.Lines) {
			for(rule in lines.Rule) {
				Rule currRule = createRule((String)rule.@type, (String)rule, types);
				if(currRule != null) {
					rulesQueue.add(currRule);
					//ZfReport.info("Rule added to rules Queue. Type - " + (String)rule.@type + ". Rule - " + (String)rule);
				}
				
			}
		}
		
		
	}
	
	
	
	/**
	 * Creates rule
	 * 
	 * @param name - name of rule
	 * @param regex - regular expression for rule
	 * @param types - type of rule
	 * 
	 * @return rule object
	 */
	private Rule createRule(String name, String regex, RuleTypes types) {
		Rule rule = null;
		
		RuleType type = types.getRuleTypeByName(name);
		
		if(type == null) {
			return rule;
		}
		
		rule = new Rule(type, regex);
		
		return rule;
	}
	
	
	
	public ArrayList<Rule> getRules() {
		return rulesQueue;
	}

	public ArrayList<Delimiter> getDelimiters() {
		return delimQueue;
	}



	
}