package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Company    : ZF Engineering, Pilsen, Czech Republic
//Department : DITIS
//Project    : build_log_parser Jenkins
//Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
//Class for sector from document.
//
//------------------------------------------------------------------------------

import java.util.ArrayList;

public class Sector {
	//Queue with all lines in this sector
	private ArrayList<Line> linesQueue = new ArrayList<Line>();
	
	//Type of sector
	private RuleType type = null;
	//References to this sector
	private String ref = null;
	//Sector start line number
	private int startLineNumber;
	//Sector end line number
	private int endLineNumber;
	
	/**
	 * Constructor for common sector
	 * 
	 * @param linesQueue - list of lines
	 * @param startLineNumber - start line number
	 * @param endLineNumber - end line number
	 */
	public Sector(ArrayList<Line> linesQueue, int startLineNumber, int endLineNumber) {
		this.startLineNumber = startLineNumber;
		this.endLineNumber = endLineNumber;
		this.linesQueue = linesQueue;
	}
	
	/**
	 * Constructor for error sector
	 * 
	 * @param linesQueue - list of lines
	 * @param type - message type
	 * @param ref - reference to sector
	 * @param startLineNumber - start line number
	 * @param endLineNumber - end line number
	 */
	public Sector(ArrayList<Line> linesQueue, RuleType type, String ref, int startLineNumber, int endLineNumber) {
		this(linesQueue, startLineNumber, endLineNumber);
		this.type = type;
		this.ref = ref;
		
	}
	
	public RuleType getType() {
		return type;
	}

	public ArrayList<Line> getContent() {
		return linesQueue;
	}

	public String getRef() {
		return ref;
	}

	
	public int getStartLineNumber() {
		return startLineNumber;
	}
	
	public int getEndLineNumber() {
		return endLineNumber;
	}
	
	/**
	 * Return content in info format
	 * @return
	 */
	public String getInfo() {
		String refr = "COMMON";
		if(ref != null) {
			refr = ref;
		}
		
		String content = "";
		
		for(Line line: linesQueue) {
			content += "[" + refr + "]" + " " + line.getContent() + "\n";
		}
		
		if(content.length() > 0) {
			content = content.substring(0, content.length() - 1);
		}
		
		return content;
	}
	
	/**
	 * Return info about sector without references 
	 * @return
	 */
	public String getInfoWithoutRef() {
		String refr = "COMMON";
		if(ref != null) {
			refr = ref;
		}
		
		String content = "";
		
		for(Line line: linesQueue) {
			content += line.getContent() + "\n";
		}
		
		if(content.length() > 0) {
			content = content.substring(0, content.length() - 1);
		}
		
		return content;
	}
}
