package bootloader

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : bootloader Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/**
 * This is a new version of the main script of the project. It used to run build.
 * At the end the script copy results files to the share.
 */

import project_lib.ProjectUtils
import lib_groovy.*
import project_lib.CMTChecker
import project_lib.VariantConfig

import java.nio.file.Path
import java.nio.file.Paths

//Map with the script configuration
Map	configMap
//Type from script parameter (debug, release)
String type
//AURIX version
String aurixV
//Component type
String componentType
//Disk letter of the view
String resultLetter

try{
    //AURIX version
    aurixV = System.getenv("AURIXV")
    //Loads build component type from script parameter
    componentType = args[0]
    //Loads build type from script parameter (debug, release)
    type = args[1].toLowerCase()

    String pathToConfig = Paths.get(System.getenv("scriptFolder")).resolve("config\\bootloaderConfig" + aurixV.toUpperCase() + ".xml").toString()

    configMap = ProjectUtils.loadAndValidateConfig(pathToConfig, type)

    Path resultDirPath = Paths.get(ProjectUtils.foundCreatedDir(configMap["resultDir"]))

    resultDirPath = resultDirPath.resolve(type)

    //Creates and starts result view
    viewName = configMap["viewName"].replace("%COMPUTERNAME%", System.getenv("COMPUTERNAME")).replace("%TYPE%", type.toUpperCase()) + "_" + componentType.toUpperCase()

    ClearCase.createView(viewName, false, resultDirPath.resolve(configMap["configSpecTime"]).toString())

    //Mount required vobs
    ClearCase.mountMultipleVobs(configMap["requiredVobs"])


    //Check if result view is mounted to drive letter and mounts it if its not.
    try{
        resultLetter = Utils.mountViewToLetter(viewName, false)
    }
    catch(Exception e){
        Utils.unmountDiskLetter(resultLetter, false)
        System.exit(5)
    }

    Path pathToDrive = Paths.get(resultLetter)

    //Path outputDirPath = pathToDrive.resolve(configMap["outputDir"])


    //Creates buildLogs.txt for storing logs of the build.
    Path logPath = resultDirPath.resolve(componentType).resolve("buildLogs.txt")
    File logsFile = new File(logPath.toString())
    logsFile.getParentFile().mkdirs()


    String pathToCompConfig = Paths.get(System.getenv("scriptFolder")).resolve("config\\bootloaderConfig" + aurixV.toUpperCase() + componentType.toUpperCase() + ".xml").toString()

    Map	componentTypeConfig = ProjectUtils.loadAndValidateConfig(pathToCompConfig, type)

    String[] components = componentTypeConfig["components"].split(",")

    for(String component: components){
        component = component.trim()

        Map	compConfig = ProjectUtils.loadAndValidateConfig(pathToCompConfig, component)
        //Prepares variables for calling build batch

        if(compConfig == null){
            continue
        }

        String[] batches = openBatches(compConfig["batch"], resultLetter)

        //Runs build
        String pathToBatch
        String batchName
        String script
        Map procResult
        String errorLevelOk = "Error level is 0"
        String errorTrap = "It's a trap"

        for(String batch: batches) {

            int index = batch.lastIndexOf("\\")
            pathToBatch = batch.substring(0, index)
            batchName = batch.substring(index + 1, batch.length())

            //Prepare script for running build.
            script = "cmd /c cd /D ${pathToBatch} & call ${batchName} -a"
            //Runs build.
            procResult = WinUtils.execute(script)
            logsFile << procResult["text"].toString() + "\n"

            if(!procResult["text"].toString().contains(errorLevelOk)) {continue}

            //Prepare script for running build with parameters.
            script = "cmd /c cd /D ${pathToBatch} & call ${batchName} lint -a"
            //Runs build.
            procResult = WinUtils.execute(script)
            logsFile << procResult["text"].toString() + "\n"

            int i = 0
            int minWaitTime = 1000*60*10
            int maxWaitTime = 1000*60*15
            int currentWaitTime

            while(i < 3) {
                //Prepare script for running build with parameters.
                script = "cmd /c cd /D ${pathToBatch} & call ${batchName} cmtexcel -a"
                //Runs build.
                procResult = WinUtils.execute(script)
                logsFile << procResult["text"].toString() + "\n\n"

                if(!procResult["text"].toString().contains(errorTrap)) { break }

                String timeStamp = ProjectUtils.getTimeStamp()
                String message = timeStamp +  ": CMT part was not builded\n"; def dst = new File(configMap["cmtError"])
                dst << message

                String cmtLog = ""
                procResult["text"].toString().eachLine{
                    String line = "		" + it + "\n"
                    cmtLog += line
                }

                dst << cmtLog

                i++

                currentWaitTime = Math.abs( new Random().nextInt() % (maxWaitTime - minWaitTime) ) + minWaitTime
                ZfReport.info("i = " + i)
                ZfReport.info("Script will wait ${currentWaitTime} milliseconds or ${currentWaitTime/1000} seconds, or ${currentWaitTime/1000/60} minutes")

                sleep(currentWaitTime)
                ZfReport.info("Script woke up")
            }


            //Prepare script for running build with parameters.
            script = "cmd /c cd /D ${pathToBatch} & call ${batchName} make_check"
            //Runs build.
            procResult = WinUtils.execute(script)

            logsFile << procResult["text"].toString() + "\n"


            String makeCheckResult = "passed"

            if(!procResult["text"].toString().contains("No problems found")) {
                makeCheckResult = "failed"
            }

            addToBuildResultXML(makeCheckResult, resultDirPath.resolve(componentType) , "MAKE_" + component)
        }

        //run ram_rom.bat fo ZFFD
        if(compConfig.containsKey("additionalBatch") && !compConfig["additionalBatch"].empty) {

            String[] addBatches = openBatches(compConfig["additionalBatch"], resultLetter)

            for(String batch: addBatches) {
                int index = batch.lastIndexOf("\\")
                pathToBatch = batch.substring(0, index)
                batchName = batch.substring(index + 1, batch.length())

                //Prepare script for running build.
                script = "cmd /c cd /D ${pathToBatch} & call ${batchName} -a"
                //Runs build.
                procResult = WinUtils.execute(script)
                logsFile << procResult["text"].toString() + "\n"
            }

        }

        //Copies filest from output directory to result
        copyFiles(compConfig, configMap, pathToDrive, resultDirPath, component, componentType)
    }


    try{
        //Unmount disc letter
        Utils.unmountDiskLetter(resultLetter, false)
        if(type == "release") {
            ClearCase.removeView(viewName)
        }
    } catch (Exception ex) {
        ProjectUtils.sendMail(configMap["EmailRecipientsFatal"].toString(), "Remove view problem " + (configMap['projectName']).toUpperCase(), ex.getMessage())
    }

    return


} catch(Exception e) {
    ZfReport.fatal(e.message)
    ZfReport.fatal(e.stackTrace)

    if (configMap != null) {
        ProjectUtils.sendMailFatal(configMap["EmailRecipientsFatal"])

        try{
            Utils.unmountDiskLetter(resultLetter, false)
            if(type == "release") {
                ClearCase.removeView(viewName)
            }
        } catch ( Exception ex ) {
            ZfReport.fatal(ex.message)
            ZfReport.fatal(ex.stackTrace)
        }
    }

    System.exit(1)
}

/**
 * Creates batches file objects
 *
 * @param batches - contains paths to batches
 * @param resultLetter - letter on that was view mounted
 *
 * @return file objects
 */
private static def openBatches(String batches, String resultLetter) {
    String[] pathes = batches.split(",")

    String[] files = new String[pathes.length]

    for(int i=0; i<pathes.length; i++) {
        files[i] = "${resultLetter}${pathes[i]}"
    }

    return files
}

/**
 * Creates or appends to result xml
 *
 * @param value - current value
 * @param path - path to the result xml
 * @param name - name
 */
private static void addToBuildResultXML(String value, Path path, String name) {
    String outputPath = path.resolve("build_result.xml").toString()
    File makeCheckXML = new File(outputPath)

    if(!makeCheckXML.exists()) {
        makeCheckXML.createNewFile()
    }

    XmlStorage makeCheckXmlStorage = new XmlStorage(outputPath, false)

    try {
        makeCheckXmlStorage.loadData()
    } catch (Exception e) {}

    makeCheckXmlStorage.setValue(name, value)

    makeCheckXmlStorage.saveData()

}

/**
 * Copies files from View to the share and creates CMT checker
 *
 * @param compConfig - Map with the component configuration
 * @param configMap - Map with the script configuration
 * @param pathToDrive - drive letter
 * @param resultDirPath - path to the share
 * @param component - component name
 * @param componentType - ZF Boot
 *
 */
private static void copyFiles(Map compConfig, Map configMap, Path pathToDrive, Path resultDirPath, String component, String componentType){
    //Reads configuration file with min/max values
    Map cmtCheckerConfig = VariantConfig.readConfigFile(System.getenv("cmtCheckerConfig"))


    String[] outputFilesNames = compConfig["outputFilesCopy"].split(",")


    //Path to output build directory
    Path outputPath = pathToDrive.resolve(compConfig["outputDir"] + "").resolve(compConfig["outputPostfix"] + "")
    //Path to directory, in which will be copy results of build
    Path resultPath = resultDirPath.resolve(componentType).resolve(compConfig["outputPostfix"] + "")

    //path to zip folder
    Path pathToZip = resultDirPath.resolve(configMap["zipName"]  + "").resolve(componentType).resolve(compConfig["outputPostfix"] + "")
    //new File(pathToZip.toString()).mkdirs()
    String[] filesToZip = compConfig["zipFiles"].split(",")

    //Iterates through files
    for(int k=0; k<outputFilesNames.length; k++) {


        String pathToCurrFile = outputPath.resolve(outputFilesNames[k]).toString()
        File currFile = new File(pathToCurrFile)

        //If current file does not exist, then new iteration
        if(!currFile.exists()) {
            ZfReport.info("File ${pathToCurrFile} does not exist.")
            continue
        }

        //Copies current file from output build folder to the result directory (from View to share)
        Utils.copy(outputPath.toString(), resultPath.toString(), outputFilesNames[k])

        if(filesToZip.contains(outputFilesNames[k])){
            Utils.copy(outputPath.toString(), pathToZip.toString(), outputFilesNames[k])
        }

        //In case existing cmt folder, creates CMT checker HTML file
        if(outputFilesNames[k].contains("cmt.xls")) {

            Path sourcePath = Paths.get(System.getenv("SOURCEPATH"))

            //if exists configuration file for the current variant, uses it. Otherwise, use default configuration
            String pathToCMTVariantConfig = sourcePath.resolve("variants_cfg\\cmt" + compConfig["variant"].toUpperCase() + "Config.xml")

            Map cmtConfig

            if((new File(pathToCMTVariantConfig)).exists()) {
                cmtConfig = VariantConfig.readConfigFile(pathToCMTVariantConfig)
            } else {
                cmtConfig = cmtCheckerConfig
            }


            //Path to the builded CMT file
            String pathToCmtMapNew = pathToDrive.resolve(compConfig["newCMTFile"] + "").toString()

            //Path to the old XMT file
            String pathToCmtMapOld = pathToDrive.resolve(compConfig["oldCMTFile"] + "").toString()

            //In case if some of this files does not exist, goes to next iteration
            if(!(new File(pathToCmtMapNew)).exists() || !(new File(pathToCmtMapOld)).exists()) {
                ZfReport.info("Some cmt file does not exist")
                continue
            }

            //Parses CMT files
            def cmtMapNew = CMTChecker.parse(pathToCmtMapNew)
            def cmtMapOld = CMTChecker.parse(pathToCmtMapOld)

            //Creates newAllTracked part
            String newAllTracked = CMTChecker.countMetrixes(cmtMapNew, cmtConfig, sourcePath.resolve("tmpl\\newAllTracked.html").toString())
            //Creates changesTracked part
            String changesTracked = CMTChecker.compareNewAndOld(cmtMapNew, cmtMapOld, cmtConfig, sourcePath.resolve("tmpl\\changesTrackedTmpl.html").toString())

            //Creates new and old CMT tables
            String newAll = CMTChecker.allMapToHtmlTable(cmtMapNew, sourcePath.resolve("tmpl\\newAll.html").toString())
            String oldAll = CMTChecker.allMapToHtmlTable(cmtMapOld, sourcePath.resolve("tmpl\\oldAll.html").toString())

            //Creates full html
            String mainHtml = CMTChecker.createMainHtml(newAllTracked, changesTracked, oldAll, newAll, sourcePath.resolve("tmpl\\mainTmpl.html").toString())

            String cmtCheckerPath = resultPath.resolve("CMT_" + componentType.toUpperCase() + "_checker.html").toString()

            //Writes html file to result folder
            CMTChecker.writeToHtml(cmtCheckerPath, mainHtml)
            ZfReport.info("Created: " + cmtCheckerPath)

            addToBuildResultXML(CMTChecker.isOk(),  resultDirPath.resolve(componentType), "CMT_" + component)

        }
    }


}