//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : bootloader Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/**
 * This script deletes old result folders
 */

package bootloader

import project_lib.ProjectUtils
import lib_groovy.*

import java.nio.file.Paths

/////////////////////////////////////////////////SCRIPT VARIABLES/////////////////////////////////////////////

//Map with the script configuration
Map	configMap

/////////////////////////////////////////////////////RUN//////////////////////////////////////////////////////

try{
	//AURIX version
	String aurixV = System.getenv("AURIXV")
	//Loads build type from script parameter (debug, release)
	String type = args[0]

	//Load configuration
	String pathToConfig = Paths.get(System.getenv("scriptFolder")).resolve("config\\bootloaderConfig" + aurixV.toUpperCase() + ".xml").toString()

	configMap = ProjectUtils.loadAndValidateConfig(pathToConfig, type)
	
	File f = new File(System.getenv("PATH_TO_LICENSE_FILE"))

	if(f.exists() && !f.isDirectory()) { 
	    f.delete()
	}
	
	//Result directory
	File dir = new File(configMap["resultDir"])
	
	//List of folder inside result directory
	List listOfDir = []
	
	//Creates list of folders
	dir.eachDir{ file -> listOfDir << file}
	
	//How match result folder can exist
	int resultFolderCount = Integer.parseInt(configMap["ResultFolderCount"])

	//How match result folder should be delete
	int shouldBeDeleted = listOfDir.size() - resultFolderCount
	
	//Deletes result folders
	if(shouldBeDeleted > 0) {
		listOfDir.sort{it.lastModified()}
		
		listOfDir.take(shouldBeDeleted).each {
			boolean isDeleted = it.deleteDir()
			isDeleted ? ZfReport.info("Folder ${it.path} deleted!") : ZfReport.info("Folder ${it.path} did not delete!")
		}
	}
} catch ( Exception e ) {
	ZfReport.fatal(e)
	System.exit(1)
}