//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : bootloader Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/**
 *This script creates directory for new build
 */

package bootloader

import java.nio.file.Path

import java.text.SimpleDateFormat
import java.nio.file.Paths

import project_lib.ProjectUtils
import lib_groovy.*

//Map with the script configuration
Map	configMap
//Info about last build
XmlStorage myXmlStorage

/////////////////////////////////////////////////SCRIPT VARIABLES/////////////////////////////////////////////

try {
	//AURIX version
	String aurixV = System.getenv("AURIXV")
	//Loads build type from script parameter (debug, release)
	String type = args[0]

	//Load configuration
	String pathToConfig = Paths.get(System.getenv("scriptFolder")).resolve("config\\bootloaderConfig" + aurixV.toUpperCase() + ".xml").toString()

	configMap = ProjectUtils.loadAndValidateConfig(pathToConfig, type)
	
	//Creates file with paths to the license servers
	String[] licLines = System.getenv("LICENSE_CONTENT").split("-------")
	
	String lic = ""
	
	for(String line: licLines) {
		lic += line + "\r\n"
	}
	
	new File(System.getenv("PATH_TO_LICENSE_FILE")).text = lic


	//Get current time in format for output folder
	SimpleDateFormat ft = new SimpleDateFormat ("dd-MM-YYYY.HH-mm-ss")
	Date dNow = new Date()
	
	String time = ft.format(dNow)

	Path resultDirFronConfig = Paths.get(configMap["resultDir"])

	//Prepares path variables for the build result directory
	Path resultDirPath = resultDirFronConfig.resolve(time).resolve(type)
	
	File dir = new File(resultDirPath.toString())
	
	dir.mkdirs()
	
	ZfReport.info("Directory " + resultDirPath.toString() + " created")
	
	SimpleDateFormat ftConfigSpec = new SimpleDateFormat ("dd-MMM-YYYY.HH:mm:ss", Locale.ENGLISH)
	
	String csTime = ftConfigSpec.format(dNow) + "UTC+02:00"
	
	String pathToCS = resultDirFronConfig.resolve(configMap["configSpecTime"]).toString()
	
	//Load last builded version from lastJenkins.txt file
	myXmlStorage = new XmlStorage(resultDirFronConfig.resolve(configMap['lastJenkins']).toString(), true)
	myXmlStorage.loadData()
	
	String configSpec = ProjectUtils.createConfigSpec(pathToCS, csTime, myXmlStorage.getValue("version"))
	
	String configSpecPath = resultDirPath.resolve(configMap["configSpecTime"]).toString()

	new File(configSpecPath).text = configSpec
	
	
	ZfReport.info("File " + configSpecPath + " created")
	
} catch(Exception e) {
	ZfReport.fatal(e)
	
	System.exit(1)
}
