package project_lib

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : bootloader Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


//------------------------------------------------------------------------------
// This class checks lint log files
//------------------------------------------------------------------------------

import lib_groovy.ZfReport
import lib_groovy.*

class LintCompare{

	static Map dbResults = [:]
	static int lintErr = 0
	static int lintWrn = 0
	static int lintSum = 0

	/**
	 * Parse ERROR, WARNING and SUM values from a given lint statistics file
	 * @par lintLog - log file text
	 * @return values for a given info types separated by semicolon
	 */
	public static void parseLint(String lintLog) {
		lintErr = 0
		lintWrn = 0
		lintSum = 0
		
		def result = ""
		def logLines = lintLog.split("\\r\\n|\\n|\\r")
		def pattern = ".*(?i)(ERROR|WARNING|SUM).*"

		for (int i = 0 i < logLines.size() i++) {

			def match = logLines[i] =~ pattern

			if (match != null && match.size() > 0) {

				result += match[0][0] + ""
			}
		}

		if(result.toString().length() != 0) {
			def lintRes = result.split("")
			for (int i = 0 i < lintRes.size() i++) {
				String loverCaseLine = lintRes[i].toLowerCase()
				
				
				if(loverCaseLine.contains("error")) {
					lintErr++
				} else if(loverCaseLine.contains("warning")) {
					lintWrn++
				} else if(loverCaseLine.contains("sum")) {
					lintSum++
				}
				
			}
	
		}
		

	}

	/**
	 * Parse ERROR, WARNING count from a given error file
	 * @par logFileText - log file text
	 * @return values for a given info types separated by semicolon
	 */
	public static void parseErrors(String logFileText) {

		def logLines = logFileText.split("\\r\\n|\\n|\\r")
		def pattern = "^(cptc|ctc|astc) ([WEF]).*"
		int wrn = 0
		int err = 0

		for (int i = 0 i < logLines.size() i++) {

			def match = logLines[i] =~ pattern

			if (match != null && match.size() > 0) {

				switch (match[0][2]) {
					case "W":
						wrn++
						break
					case "E":
					case "F":
						err++
						break
					default:
						break
				}
			}
		}

		dbResults["ctcerr"]=err
		dbResults["ctcwarn"]=wrn

	}
	
	
	public static int getLintErr() {
		return lintErr
	}
	
	public static int getLintWrn() {
		return lintWrn
	}
	
	public static int getLintSum() {
		return lintSum
	}
}